'use strict';

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _kea = require('kea');

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _index = require('../index');

var _index2 = _interopRequireDefault(_index);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

beforeEach(function () {
  (0, _kea.resetKeaCache)();
  (0, _kea.activatePlugin)(_index2.default);
}); /* global test, expect, beforeEach */


test('thunks work', function () {
  var store = (0, _kea.getStore)();

  var thunkRan = false;

  var firstLogic = (0, _kea.kea)({
    path: function path() {
      return ['scenes', 'thunks', 'first'];
    },
    actions: function actions(_ref) {
      var constants = _ref.constants;
      return {
        updateName: function updateName(name) {
          return { name: name };
        }
      };
    },
    thunks: function thunks(_ref2) {
      var actions = _ref2.actions,
          dispatch = _ref2.dispatch,
          getState = _ref2.getState;
      return {
        updateNameAsync: function updateNameAsync(name) {
          thunkRan = true;
          actions.updateName(name);
        }
      };
    },
    reducers: function reducers(_ref3) {
      var _ref4;

      var actions = _ref3.actions,
          constants = _ref3.constants;
      return {
        name: ['chirpy', _propTypes2.default.string, (_ref4 = {}, _ref4[actions.updateName] = function (state, payload) {
          return payload.name;
        }, _ref4)]
      };
    }
  });

  expect(firstLogic._keaPlugins.thunk).toBe(true);
  expect(firstLogic._isKeaFunction).toBe(true);
  expect(firstLogic._isKeaSingleton).toBe(true);
  expect((0, _keys2.default)(firstLogic.actions)).toEqual(['updateName', 'updateNameAsync']);
  expect((0, _keys2.default)(firstLogic.selectors).sort()).toEqual(['name', 'root']);

  store.dispatch(firstLogic.actions.updateNameAsync('derpy'));
  expect(firstLogic.selectors.name(store.getState())).toBe('derpy');

  expect(thunkRan).toBe(true);
});

test('thunks can call thunks', function () {
  var store = (0, _kea.getStore)();

  var firstThunkRan = false;
  var secondThunkRan = false;

  var firstLogic = (0, _kea.kea)({
    path: function path() {
      return ['scenes', 'thunks', 'first'];
    },
    actions: function actions(_ref5) {
      var constants = _ref5.constants;
      return {
        updateName: function updateName(name) {
          return { name: name };
        }
      };
    },
    thunks: function thunks(_ref6) {
      var actions = _ref6.actions,
          dispatch = _ref6.dispatch,
          getState = _ref6.getState;
      return {
        updateNameAsync: function updateNameAsync(name) {
          firstThunkRan = true;
          actions.updateName(name);
        },
        updateNameReallyAsync: function updateNameReallyAsync(name) {
          secondThunkRan = true;
          actions.updateNameAsync(name);
        }
      };
    },
    reducers: function reducers(_ref7) {
      var _ref8;

      var actions = _ref7.actions,
          constants = _ref7.constants;
      return {
        name: ['chirpy', _propTypes2.default.string, (_ref8 = {}, _ref8[actions.updateName] = function (state, payload) {
          return payload.name;
        }, _ref8)]
      };
    }
  });

  expect(firstLogic._keaPlugins.thunk).toBe(true);
  expect((0, _keys2.default)(firstLogic.actions)).toEqual(['updateName', 'updateNameAsync', 'updateNameReallyAsync']);
  expect((0, _keys2.default)(firstLogic.selectors).sort()).toEqual(['name', 'root']);

  store.dispatch(firstLogic.actions.updateNameReallyAsync('derpy'));
  expect(firstLogic.selectors.name(store.getState())).toBe('derpy');

  expect(firstThunkRan).toBe(true);
  expect(secondThunkRan).toBe(true);
});

test('connected thunks work', function () {
  var store = (0, _kea.getStore)();

  var thunkRan = false;

  var firstLogic = (0, _kea.kea)({
    path: function path() {
      return ['scenes', 'thunks', 'first'];
    },
    actions: function actions(_ref9) {
      var constants = _ref9.constants;
      return {
        updateName: function updateName(name) {
          return { name: name };
        }
      };
    },
    thunks: function thunks(_ref10) {
      var actions = _ref10.actions,
          dispatch = _ref10.dispatch,
          getState = _ref10.getState;
      return {
        updateNameAsync: function updateNameAsync(name) {
          thunkRan = true;
          actions.updateName(name);
        }
      };
    },
    reducers: function reducers(_ref11) {
      var _ref12;

      var actions = _ref11.actions,
          constants = _ref11.constants;
      return {
        name: ['chirpy', _propTypes2.default.string, (_ref12 = {}, _ref12[actions.updateName] = function (state, payload) {
          return payload.name;
        }, _ref12)]
      };
    }
  });

  var secondLogic = (0, _kea.kea)({
    connect: {
      actions: [firstLogic, ['updateNameAsync']],
      props: [firstLogic, ['name']]
    }
  });

  expect(firstLogic._keaPlugins.thunk).toBe(true);
  expect((0, _keys2.default)(secondLogic.actions)).toEqual(['updateNameAsync']);
  expect((0, _keys2.default)(secondLogic.selectors).sort()).toEqual(['name']);

  store.dispatch(secondLogic.actions.updateNameAsync('derpy'));
  expect(secondLogic.selectors.name(store.getState())).toBe('derpy');

  expect(thunkRan).toBe(true);
});

test('async works', function () {
  var store = (0, _kea.getStore)();

  var actionsRan = [];

  var instantPromise = function instantPromise() {
    return new _promise2.default(function (resolve) {
      actionsRan.push('in promise');
      resolve();
    });
  };

  var asyncLogic = (0, _kea.kea)({
    path: function path() {
      return ['scenes', 'thunks', 'async'];
    },
    actions: function actions(_ref13) {
      var constants = _ref13.constants;
      return {
        updateName: function updateName(name) {
          return { name: name };
        }
      };
    },
    thunks: function thunks(_ref14) {
      var actions = _ref14.actions,
          selectors = _ref14.selectors,
          get = _ref14.get,
          fetch = _ref14.fetch,
          dispatch = _ref14.dispatch,
          getState = _ref14.getState;
      return {
        updateNameAsync: function () {
          var _ref15 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee(name) {
            return _regenerator2.default.wrap(function _callee$(_context) {
              while (1) {
                switch (_context.prev = _context.next) {
                  case 0:
                    actionsRan.push('before promise');
                    _context.next = 3;
                    return instantPromise();

                  case 3:
                    actionsRan.push('after promise');

                    _context.next = 6;
                    return actions.anotherThunk();

                  case 6:
                    actions.updateName(name);

                    expect(selectors.name(getState())).toEqual('derpy');
                    expect(get('name')).toEqual('derpy');
                    expect(fetch('name')).toEqual({ name: 'derpy' });

                  case 10:
                  case 'end':
                    return _context.stop();
                }
              }
            }, _callee, undefined);
          }));

          function updateNameAsync(_x) {
            return _ref15.apply(this, arguments);
          }

          return updateNameAsync;
        }(),
        anotherThunk: function () {
          var _ref16 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee2() {
            return _regenerator2.default.wrap(function _callee2$(_context2) {
              while (1) {
                switch (_context2.prev = _context2.next) {
                  case 0:
                    actionsRan.push('another thunk ran');

                  case 1:
                  case 'end':
                    return _context2.stop();
                }
              }
            }, _callee2, undefined);
          }));

          function anotherThunk() {
            return _ref16.apply(this, arguments);
          }

          return anotherThunk;
        }()
      };
    },
    reducers: function reducers(_ref17) {
      var _ref18;

      var actions = _ref17.actions,
          constants = _ref17.constants;
      return {
        name: ['chirpy', _propTypes2.default.string, (_ref18 = {}, _ref18[actions.updateName] = function (state, payload) {
          return payload.name;
        }, _ref18)]
      };
    }
  });
  actionsRan.push('before action');

  return store.dispatch(asyncLogic.actions.updateNameAsync('derpy')).then(function () {
    actionsRan.push('after dispatch');
    expect(asyncLogic.selectors.name(store.getState())).toBe('derpy');
    actionsRan.push('after action');

    expect(actionsRan).toEqual(['before action', 'before promise', 'in promise', 'after promise', 'another thunk ran', 'after dispatch', 'after action']);
  });
});